/* FILE: timeevolver.h                 -*-Mode: c++-*-
 *
 * Abstract time evolver class
 *
 */

#ifndef _OXS_TIMEEVOLVER
#define _OXS_TIMEEVOLVER

#include "evolver.h"
#include "output.h"

/* End includes */

class Oxs_TimeDriver; // Forward references
struct Oxs_DriverStepInfo;

class Oxs_TimeEvolver:public Oxs_Evolver {
private:
  UINT4m energy_calc_count; // Number of times GetEnergyDensity
  /// has been called in current problem run.

  Oxs_MeshValue<REAL8m> temp_energy;     // Scratch space used by
  Oxs_MeshValue<ThreeVector> temp_field; // GetEnergyDensity().

  // Outputs maintained by this interface layer.  These are conceptually
  // public, but are specified private to force clients to use the
  // output_map interface.
  void UpdateEnergyOutputs(const Oxs_SimState&);
  void FillEnergyCalcCountOutput(const Oxs_SimState&);
  Oxs_ScalarOutput<Oxs_TimeEvolver> total_energy_output;
  Oxs_VectorFieldOutput<Oxs_TimeEvolver> total_field_output;
  Oxs_ScalarOutput<Oxs_TimeEvolver> energy_calc_count_output;

  // Disable copy constructor and assignment operator by declaring
  // them without defining them.
  Oxs_TimeEvolver(const Oxs_TimeEvolver&);
  Oxs_TimeEvolver& operator=(const Oxs_TimeEvolver&);

protected:
  Oxs_TimeEvolver(const char* name,      // Child instance id
                 Oxs_Director* newdtr);  // App director
  Oxs_TimeEvolver(const char* name,
                 Oxs_Director* newdtr,
                 const char* argstr);      // MIF block argument string

  virtual BOOL Init();  // All children of Oxs_TimeEvolver *must*
  /// call this function in their Init() routines.  The main purpose
  /// of this function is to initialize output variables.

  void GetEnergyDensity(const Oxs_SimState& state,
			Oxs_MeshValue<REAL8m>& energy,
			Oxs_MeshValue<ThreeVector>* mxH_req,
			Oxs_MeshValue<ThreeVector>* H_req,
			REAL8m& pE_pt);
public:
  virtual ~Oxs_TimeEvolver() {}

  virtual  BOOL
  Step(const Oxs_TimeDriver* driver,
       Oxs_ConstKey<Oxs_SimState> current_state,
       const Oxs_DriverStepInfo& step_info,
       Oxs_Key<Oxs_SimState>& next_state) = 0;
  // Returns true if step was successful, false if
  // unable to step as requested.  The evolver object
  // is responsible for calling driver->FillState()
  // and driver->FillStateSupplemental() to fill
  // next_state as needed.
};

#endif // _OXS_TIMEEVOLVER
