/* FILE: nb.cc                   -*-Mode: c++-*-
 *
 *	The Nuts & Bolts extension.
 *
 *	This extension provides a collection of C++ classes and functions
 * of general utility to all OOMMF applications.
 * 
 * NOTICE: Please see the file ../../LICENSE
 *
 * Last modified on: $Date: 2004/09/02 18:03:11 $
 * Last modified by: $Author: dgp $
 */

/* Header file for this extension */
#include "oc.h"
#include "evoc.h"
#include "nb.h"

/* End includes */     

int 
Nb_Init(Tcl_Interp *interp)
{
#define RETURN_TCL_ERROR                                               \
    Tcl_AddErrorInfo(interp, ab1("\n    (in Nb_Init())"));             \
    return TCL_ERROR

  Oc_AutoBuf ab,ab1,ab2,ab3,ab4;

  if (Tcl_PkgPresent(interp, ab("Oc"), ab1("1.2.0.4"), 0) == NULL) {
    Tcl_AppendResult(interp, ab("\n\t(Nb "NB_VERSION" needs Oc 1.2.0.4)"), NULL);
    RETURN_TCL_ERROR;
  }

  // Register C++-based Tcl commands of this extension
  Oc_RegisterCommand(interp,"Nb_ComputeCRCBuffer",
		     (Tcl_CmdProc *)NbComputeCRCBufferCmd);
  Oc_RegisterCommand(interp,"Nb_ComputeCRCChannel",
		     (Tcl_CmdProc *)NbComputeCRCChannelCmd);
  Oc_RegisterCommand(interp,"Nb_Gcd",(Tcl_CmdProc *)NbGcdCmd);
  Oc_RegisterCommand(interp,"Nb_RatApprox",(Tcl_CmdProc *)NbRatApprox);
  Oc_RegisterCommand(interp,"Nb_GetColor",(Tcl_CmdProc *)NbGetColor);
  Oc_RegisterCommand(interp,"Oc_Srand",(Tcl_CmdProc *)OcSrand);
  Oc_RegisterCommand(interp,"Oc_UnifRand",(Tcl_CmdProc *)OcUnifRand);
  Oc_RegisterCommand(interp,"Oc_Times",(Tcl_CmdProc *)OcTimes);

  // Add Tcl expr command extensions, if any.
  Oc_AddTclExprExtensions(interp);
  Oc_RegisterCommand(interp,"Oc_AddExprExtensions",
		     (Tcl_CmdProc *)OcAddTclExprExtensions);

  if (Tcl_PkgProvide(interp, ab("Nb"), ab1(NB_VERSION)) != TCL_OK) {
    RETURN_TCL_ERROR;
  }

  // Setup Tcl script portion of Nb extension
  if ( Oc_InitScript(interp, "Nb", NB_VERSION) != TCL_OK) {
    RETURN_TCL_ERROR;
  }

  return TCL_OK;

#undef RETURN_TCL_ERROR
}
